/* This is  -*- C++ -*-  */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///  \file  codewordEnumerator.hpp
///  \brief A class which enumerates all codewords
///  \author Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  \date   14.10.2010
///
///  Copyright (C) 2010 Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  Distributed  under  the  terms  of  the  GNU  General  Public  License (GPL)
///  http://www.gnu.org/licenses/
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef CODEWORDENUMERATOR_HPP_
#define CODEWORDENUMERATOR_HPP_

#include "vector.hpp"

class Field;
class BinaryField;
class Z4;
class F2_F2;

template<typename RING>
class Matrix;

//template<typename RING>
//struct InformationAndCode_Word
//{
//    Vec<RING> m_infword;
//    Codeword<RING> m_codeword;
//    InformationAndCode_Word() {};
//    InformationAndCode_Word(const Vec<RING>& infword, const Codeword<RING>& codeword) : m_infword(infword), m_codeword(codeword) {}
//
//    InformationAndCode_Word<RING>& operator=(const InformationAndCode_Word<RING>& rhs)
//    {
//        m_infword = rhs.m_infword;
//        m_codeword = rhs.m_codeword;
//        return *this;
//    }
//};
//
//template<>
//struct InformationAndCode_Word<BinaryField>
//{
//    Codeword<BinaryField> m_codeword;
//    InformationAndCode_Word() {};
//    InformationAndCode_Word<BinaryField>(const Codeword<BinaryField>& codeword) : m_codeword(codeword) {}
//    InformationAndCode_Word<BinaryField>& operator=(const InformationAndCode_Word<BinaryField>& rhs)
//    {
//        m_codeword = rhs.m_codeword;
//        return *this;
//    }
//};

template<typename RING>
class CodewordEnumerator
{
public:
    typedef  DenseVec<RING> CW;

private:
    unsigned int m_n, m_k, m_rows;
    std::vector<unsigned int> m_type;
    CW m_actCodeword;
    std::vector<CW > m_plus;
    std::vector<CW> m_minus;

    std::vector<int> m_max; // stores the additive order of the rows (some prime powers)
    std::vector<int> m_inf_word; // stores the information words
    std::vector<int> m_direction; // stores +1 or -1 for each element

    std::vector<int>::iterator m_maxIterBeg;
    std::vector<int>::iterator m_maxIter;
    std::vector<int>::iterator m_inf_wordIterBeg;
    std::vector<int>::iterator m_inf_wordIter;
    std::vector<int>::iterator m_directionIterBeg;
    std::vector<int>::iterator m_directionIter;

    int* p_stopFlag;

public:

    CodewordEnumerator(const std::vector<unsigned int>& type, const Matrix<RING>& mat);
    bool operator++(void);
//    inline const CW& operator*(void) const;
    inline const CW* operator->(void) const;
    inline bool isProjPointRepresentative(void) const;
//    unsigned int getWeight(void) const
//    {
//        return m_actCodeword.getWeight();
//    }

//    uchar getInformationWord(void) const;

    const CW& getCodeword(void) const
    {
    return m_actCodeword;
    }

    Vec<RING> getInformationWord(void) const;

//    inline InformationAndCode_Word<RING> get(void) const;
};

//template<typename RING>
//InformationAndCode_Word<RING> CodewordEnumerator<RING>::get(void) const
//{
//    return InformationAndCode_Word<RING>(getInformationWord(), m_actCodeword);
//}
//
//template<>
//InformationAndCode_Word<BinaryField> CodewordEnumerator<BinaryField>::get(void) const;

template<>
CodewordEnumerator<Field>::CodewordEnumerator(const std::vector<unsigned int>& type, const Matrix<Field>& mat);

template<>
CodewordEnumerator<Z4>::CodewordEnumerator(const std::vector<unsigned int>& type, const Matrix<Z4>& matrix);

template<>
CodewordEnumerator<F2_F2>::CodewordEnumerator(const std::vector<unsigned int>& type, const Matrix<F2_F2>& matrix);

template<>
CodewordEnumerator<BinaryField>::CodewordEnumerator(const std::vector<unsigned int>& type, const Matrix<BinaryField>& mat);

//template<>
//uchar CodewordEnumerator<BinaryField>::getInformationWord(void) const;


//template<>
//Vec<Field> CodewordEnumerator<Field>::getInformationWord(void) const;
//
//template<>
//Vec<Z4> CodewordEnumerator<Z4>::getInformationWord(void) const;
//

template<>
Vec<Field> CodewordEnumerator<Field>::getInformationWord(void) const;

template<>
Vec<BinaryField> CodewordEnumerator<BinaryField>::getInformationWord(void) const;

template<>
Vec<Z4> CodewordEnumerator<Z4>::getInformationWord(void) const;

template<>
Vec<F2_F2> CodewordEnumerator<F2_F2>::getInformationWord(void) const;

//template<typename RING>
//uchar CodewordEnumerator<RING>::getInformationWord(void) const
//{
//    std::cerr<<"I am not a defined function"<<std::endl;
//    exit(-1);
//}
//
//template<>
//uchar CodewordEnumerator<BinaryField>::getInformationWord(void) const;

//template<>
//bool CodewordEnumerator<BinaryField>::operator++(void);

template<typename RING>
bool CodewordEnumerator<RING>::operator++(void)
{
    m_maxIter = m_maxIterBeg;
    m_inf_wordIter = m_inf_wordIterBeg;
    m_directionIter = m_directionIterBeg;

    int sum =  *m_inf_wordIter + *m_directionIter;
    while ((sum >= *m_maxIter) || (sum < 0))
    {
        *m_directionIter = - (*m_directionIter);

        ++m_directionIter;
        ++m_inf_wordIter;
        ++m_maxIter;

        sum = *m_inf_wordIter + *m_directionIter;
    }
    *m_inf_wordIter = sum;
    if (*p_stopFlag != 0)
     {
         return false;
     }

    unsigned int i = distance(m_inf_wordIterBeg, m_inf_wordIter);
    if (*m_directionIter == 1)
    {
        m_actCodeword += m_plus[i];
    }
    else
    {
        m_actCodeword += m_minus[i];
    }
    return true;
}

//template<typename RING>
//const typename CodewordEnumerator<RING>::CW& CodewordEnumerator<RING>::operator*(void) const
//{
//    return m_actCodeword;
//}
//
template<typename RING>
const typename CodewordEnumerator<RING>::CW* CodewordEnumerator<RING>::operator->(void) const
{
    return &m_actCodeword;
}

template<>
bool CodewordEnumerator<Field>::isProjPointRepresentative(void) const;

template<>
bool CodewordEnumerator<BinaryField>::isProjPointRepresentative(void) const;

template<>
bool CodewordEnumerator<Z4>::isProjPointRepresentative(void) const;

template<>
bool CodewordEnumerator<F2_F2>::isProjPointRepresentative(void) const;
#endif /* CODEWORDENUMERATOR_HPP_ */
