/* This is  -*- C++ -*-  */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///  \file  gapCASModule.cpp
///  \brief Provides an interface to GAP.
///  \author Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  \date   2010-09-29
///
///  Copyright (C) 2010 Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  Distributed  under  the  terms  of  the  GNU  General  Public  License (GPL)
///  http://www.gnu.org/licenses/
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


#include "gapCASModule.h"
#include "groups.h"

GapCASModule::GapCASModule(char* call[], const std::string& packagesFolder) : AbstractCASModule(call)
{
	m_to<<"LoadPackage( \"guava\");"<<std::endl;
	m_to << "Read( \"" << packagesFolder<< "labra.gap\");"<<std::endl;
	
	char x [256];
	while(m_from.getline(x,256) )
	{
		if(!strcmp(x, "true"))
			break;
		if(!strcmp(x, "false"))
		{
			std::cerr<<"The package GUAVA is not available in your GAP Version;"<<std::endl<<"Exiting"<<std::endl;
			exit(-1);
		}
	}
}

GapCASModule::~GapCASModule(void)
{
	m_to<<"quit;"<<std::endl;
}

void GapCASModule::getFather(const std::string& group, std::vector<int>& father, My_Long_Int& group_order )
{
	m_to<<"getFather("<<group<<", "<<father.size()<<");;"<<std::endl;
	std::vector<int>::const_iterator endIter = father.end();
	for(std::vector<int>::iterator iter=father.begin(); iter != endIter; ++iter)
	{
		m_from>> *iter;
	}
	m_from>>group_order;
}

void GapCASModule::initTrivialSn(const std::string& group, int n)
{
	m_to<<group<<" := Group(  () );;"<<std::endl;
}

void GapCASModule::extendGroup(const std::string& group, const Permutation& newgen, int n)
{
	m_to<<group << ":= ClosureGroup("<<group<<", ";
	newgen.print(m_to,1);
	m_to<<");;"<<std::endl;
}

void GapCASModule::buildGroup(const std::string& group, const std::vector<Permutation>& newgens, int n)
{
	m_to<<group<<" := Group(  [()"; //identity '()' as a dummy to avoid test on ','
	for(unsigned int i=0; i<newgens.size(); ++i)
	{
		m_to<<",";
		newgens[i].print(m_to,1);
	}
	m_to<<"]);;"<<std::endl;
}

My_Long_Int GapCASModule::getOrder( const std::string& group )
{
    My_Long_Int order;
	m_to<<"Order("<<group<<");"<<std::endl;
	m_from>>order;
	
	return order;
}

