/* This is  -*- C++ -*-  */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///  \file  weights.cpp
///  \brief Implementations for the symmetric weight.
///  \author Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  \date   2010-09-29
///
///  Copyright (C) 2010 Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  Distributed  under  the  terms  of  the  GNU  General  Public  License (GPL)
///  http://www.gnu.org/licenses/
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "weights.hpp"
#include <limits>

template<>
std::ostream& operator<<(std::ostream& os, const SymWeight<1>& w)
{
	return os<<"["<<w.qty<<"]";
}

template<>
unsigned int Distribution<SymWeight<1u> >::getMinWeight(void) const
{
    if(map.size()<=1)
    {
        return 0;
    }
    std::map<SymWeight<1u>, unsigned int>::const_iterator iter(map.begin());
    ++iter;
    return iter->first.qty;
}
//
//template<>
//Distribution<SymWeight<1u> > Distribution<SymWeight<1u> >::getDual(unsigned int n, unsigned int k) const
//{
//    Distribution<SymWeight<1u> > dualwd;
//    Krawtchouk::Table[n].getDualDistribution(k, *this, dualwd);
//    return dualwd;
//}

//template<>
//unsigned int Distribution<SymWeight<1u> >::getDualMinDist(unsigned int n) const
//{
//    return Krawtchouk::Table[n].getDualMinDist(*this);
//}

template<>
unsigned int Distribution<SymWeight<2u> >::getMinWeight(void) const
{
    if(map.size()<=1)
    {
        return 0;
    }
    std::map<SymWeight<2u>, unsigned int>::const_iterator mapIter(map.begin());
    std::map<SymWeight<2u>, unsigned int>::const_iterator mapIterEnd(map.end());
    --mapIterEnd;
    unsigned int res=std::numeric_limits<unsigned int>::max();

    for(; mapIter != mapIterEnd; ++mapIter)
    {
        unsigned int w = mapIter->first.qty[0] + 2*mapIter->first.qty[1];
        if(w<res)
        {
            res = w;
        }
     }
    return res;
}

//template<>
//SymWeightDist<1> SymWeightDist<2>::toLeeWeightDist(void) const
//{
//	SymWeightDist<1> leeWD;
//	std::map<SymWeight<2>, unsigned int>::const_iterator iterSym=map.begin();
//	std::map<SymWeight<2>, unsigned int>::const_iterator iterSymEnd=map.end();
//
//	for(; iterSym != iterSymEnd; ++iterSym)
//	{
//		SymWeight<1> w = iterSym->first.qty[0] + 2* iterSym->first.qty[1];
//		leeWD.add(w, iterSym->second);
//	}
//	return leeWD;
//}

//template<>
//template<>
//SymWeightDist<1u>::SymWeightDist(const Matrix<Field>& mat)
//{
//    std::vector<unsigned int> type(1, mat.getRows());
//    CodewordEnumerator<Field> enumerator(type, mat);
//    do
//    {
//        add( enumerator->getWeight() );
//    }while(++enumerator);
//}
//
//template<>
//template<>
//SymWeightDist<1u>::SymWeightDist(const Matrix<BinaryField>& mat)
//{
//    std::vector<unsigned int> type(1, mat.getRows());
//    CodewordEnumerator<BinaryField> enumerator(type, mat);
//    do
//    {
//        add( enumerator->getWeight() );
//    }while(++enumerator);
//}
//
//template<>
//template<>
//SymWeightDist<2u>::SymWeightDist(const Matrix<Z4>& mat)
//{
//    std::vector<unsigned int> type(2, 0);
//    Matrix<Z4> copy( mat );
//    type[0] = copy.toOrderedBasis(false, NULL, NULL);
//    type[1] = copy.getRows() - type[0];
//    CodewordEnumerator<Z4> enumerator(type, copy);
//
//    do
//    {
//        add( enumerator->getWeight() );
//    }while(++enumerator);
//}
//
//template<>
//template<>
//SymWeightDist<2u>::SymWeightDist(const Matrix<F2_F2>& mat)
//{
//    std::vector<unsigned int> type(2, 0);
//    Matrix<F2_F2> copy( mat );
//    type[0] = copy.toOrderedBasis(false, NULL, NULL);
//    type[1] = copy.getRows() - type[0];
//    CodewordEnumerator<F2_F2> enumerator(type, copy);
//
//    do
//    {
//        add( enumerator->getWeight() );
//    }while(++enumerator);
//}
